///////////////////////////////////////////////////////////////////////////
// Canon EOS Shutter-Emulation on Arduino (Nano / Pro Mini, / ...)
//
//       25.1.2017 - 31.01.2025                       Erwin Lotter
//

// EOS 600D emulation core, to be included from ShutterEmuEOS.ino


  if (triggerEvent == evCoil1Start) {        // Start-Flanke aktiviert:
    InitEvent();
    stateIndex = 0;
    //
    // Außerhalb des LV kann:
    //  a) eine normale Auslösung vorliegen, die mit c1_L_Start behandelt wird, oder
    //  b) ein Start des LV, für den c1_L_Start die ROT-Signale zu weit ausführt.
    //  c1_L_Start stoppt nach 48 ms und wartet auf Coil 2 --> Hi. Der C2_H Zeitpunkt
    //   lässt zwischen a) und b) unterscheiden.
    //
    // Im LV war die letzte Aktion entweder
    //  - coilX_FinishLVShoot, dem wieder ein cX_H_EnterLV folgen muss, oder
    //  - cX_H_EnterLV. Danach kommt entweder
    //  	a) LV-Auslösen oder 
    //  	b) LV-Ende. Dafür ist c1_L_Start ungeeignet, deshalb c1_L_StartInLV verwenden:
    //
    if (actionPtr == c1_H_EnterLV	// c1_H_EnterLV kann auch im Coil2-Modus vorkommen, wenn die nicht angeschlossen ist
      #if defined(TRIGGER2_MODE)
      || actionPtr == c2_H_EnterLV
      #endif
    // Achtung, für Coil 1 müsste hier eigentlich kommen:				??600D
    //|| actionPtr == c1_H_FinishLVShoot
    // aber	das klappt für die 600D nicht (Err20)!!
    // Ohne diese Prüfung wird nach LV-Shoot das jetzt folgende Re-Enter-LV mit c1_L_Start 
    //  statt mit c1_L_StartInLV (erfolgreich) eingeleitet. Dadurch wird dann bei C1_H das
    //  erste c1_H_EnterLV anstatt dem zweiten (dem Re-EnterLV) ausgewählt. Die sind eh gleich.
    // Der Fehler tritt auch auf, wenn c1_L_StartInLV gleich ist wie c1_L_Start. Er besteht
    //  vmtl. darin, dass nach c1_H_FinishLVShoot das C1_H Timing ncht richtig erkannt wird.
    // besser so Cam-spezifisch def.:
    //#define IS_IN_LV 		(actionPtr == cX_H_EnterLV  ||  actionPtr == c1_H_FinishLVShoot)
    // 
    ) {
      actionPtr = c1_L_StartInLV;
//PRINTLN("c1_L_StartInLV");
    }
    else {
      actionPtr = c1_L_Start;
//PRINTLN("c1_L_Start");
    }
    NotifyIntervalTimer();
  } // if (triggerEvent == evCoil1Start) -----------------------------------------------
  
  else if (triggerEvent == evCoil1Stop) { // Coil 1 wurde inaktiv (C1_H):
    //
    // Falls eine Belichtung stattfand, sollte aktuell ein bei C2_H gestartetes LONGWAIT 
    //  laufen (entweder in c2_H_FinishShoot oder in c2_H_FinishLVShoot).
    // Das wird jetzt beendet (und die davor noch anstehenden Signale
    //  werden generiert).
    //
    #if defined(TRIGGER2_MODE)	// ############# C1_H nach C2_H: Belichtung beendet
    if (prevEvent == evCoil2Stop) {
      //
      // Die 600D generiert bei LeaveLV keinen C2 Puls.
      // Deshalb via C1_H erkennen:
      //
      if (actionPtr == c1_L_StartInLV) {		// kein C2_H (und im LV)?
	actionPtr = c1_H_LeaveLV;
	stateIndex = 0;
      }
      else {
	ContinueActions();	// Actionliste bis WAIT oder LONGWAIT abarbeiten
      }
    }
    else	// falls Coil2 nicht angeschlossen ist, kam auch kein C2_H:
    #endif //defined(TRIGGER2_MODE)
    {
      unsigned long dt = triggerTime - timingZero;		// Coil-1 Pulsdauer
      if (actionPtr == c1_L_Start) {		// (noch) nicht im LV?
	if (IsInWindow_EnterLV(dt)) {
	  actionPtr = c1_H_EnterLV;		// dt passt zu LV-Start oder Restart -- FALSCH bei best. Belichtungszeiten (600D: < 1/60s!!!)
	  //PRINTLN("c1_H_EnterLV");
	}
	else {
	  actionPtr = c1_H_FinishShoot;	// zu kurz/lang für EnterLV: reguläres (nicht zu kurzes!) Auslösen
	  //PRINTLN("c1_H_FinishShoot");
	}
      }
      else { // im LV:
	      // a) Shoot im LV endet
	      // b) Re-Enter LV endet (folgt immer nach a)
	      // c) Leave LV endet: Kann nur nach EnterLV kommen, dt ist sehr kurz
	      //
	if (IsInWindow_LeaveLV(dt)) {		// LeaveLV hat kürzesten Puls, also zuverlässig zu erkennen!
	  //PRINTLN("c1_H_LeaveLV");
	  actionPtr = c1_H_LeaveLV;
	}
	else if (IsInWindow_ReEnterLV(dt)
	  #ifdef EOS600D
		 &&  actionPtr != c1_L_StartInLV  // für 600D zusätzlich prüfen: kurze Zeiten im LV liegen unter HiLIMIT_C1_LEAVE_LV
	  #endif	  
	) {
	  //PRINTLN("c1_H_(Re)EnterLV");
	  actionPtr = c1_H_EnterLV;
	}
	else {
	  // zu lange für EnterLV: reguläres (nicht zu kurzes!) Auslösen
	  // geht für 600D schief für kurze Belichtungen. Sicher zu unterscheiden, wenn vorheriger actionPtr gespeichert wird: Der war dann nicht
	  //  c1_H_FinishLVShoot, weil danach immer c1_H_EnterLV folgt
	  //PRINTLN("c1_H_FinishLVShoot");
	  actionPtr = c1_H_FinishLVShoot;
	}
      }
      stateIndex = 0;
    }
    InitEvent();
  } // if (triggerEvent == evCoil1Stop) -----------------------------------------------------
  //
  //
  #if defined(TRIGGER2_MODE)		// Auch Coil 2 Signale
  else if (triggerEvent == evCoil2Stop) {        // Coil-2 deaktiviert: Shutter öffnet
    unsigned long dt = triggerTime - timingZero;		// Coil-2 Pulsdauer
    actionPtr = SelectFinishAction(dt);
    InitEvent();
    stateIndex = 0;
  } // if (triggerEvent == evCoil2Stop) ----------------------------------------
  #endif
