#ifndef FlashUpdate_h_
#define FlashUpdate_h_

#if defined(SPIFLASH_CS)

// Verwendung des externen Flashs:
// 'Page' = 64KB Block
// Page 0 ist für Updates reserviert
//
#define FALLBACK_FLASHPAGE    0x8000
#define FALLBACK_SIZE	      0x8000
#define SERIALOUT_FLASHPAGE   0x10000
#define PROTOCOL_FLASHPAGE    0x20000


// SPIMemory Library required! (https://github.com/Marzogh/SPIMemory)

#include<SPIFlash.h>	  // braucht ca. 330 Bytes RAM!!!! (SparkFun_SPI_SerialFlash ist ähnlich)
			  //(https://github.com/LowPowerLab/SPIFlash/tree/master evtl schlanker?)

class BootFlash : public SPIFlash {
  public:
	    BootFlash(uint8_t cs = CS) : SPIFlash(cs) {uploadState = 0;}
    int8_t  AddToUpload(int8_t c);
    void    SendError();
    bool    InitCodeUpload();
    bool    Erase32K(uint32_t addr);
    bool    HasValidUpload() const    {return uploadState > 0;}
    bool    WriteFirstWord(uint32_t addr);
    bool    ReadFirstWord(uint32_t addr);
    bool    CopyFlashToFlash(uint32_t from, uint32_t to, uint16_t size);
    bool    CompleteUpload();
    #if defined(FALLBACK_SUPPORT)
    bool    HasFallback()   	      {ReadFirstWord(FALLBACK_FLASHPAGE); return firstBytes[0] != 0xFF;}
    bool    CopyFallbackTo0x000();
    #endif

  protected:
    //uint8_t GetDigitVal(uint8_t pos) const    {return (lineBuffer[pos] - '0');}
    //uint8_t GetByteVal(uint8_t pos) const     {return GetDigitVal(pos) + (GetDigitVal(pos) << 4);}

    uint8_t GetByteCount() const      {return lineBuffer[0];}
    uint8_t GetRecordType() const     {return lineBuffer[3];}
    uint8_t* GetData()	  	      {return &lineBuffer[4];}
    uint16_t GetAddress() const	      {return (lineBuffer[1] << 8) | lineBuffer[2];}

  protected:
    uint8_t lineBuffer[1+2+1+32+1]; // Byte count + Address + Record type + Data + Checksum
    int8_t  digitCnt;
    uint8_t limit;
    uint8_t byteCount;
    uint8_t firstBytes[2];
    uint8_t checksum;
    int8_t  uploadState;
};


//
// Upload vervollständigen
// 
inline
bool BootFlash::CompleteUpload()
{
  if (WriteFirstWord(0)) {
    uploadState = 2;      // zeigt an, dass der Upload bereit zur Installation ist
    return true;
  }
  return false;
}


//
// Erstes Wort des Uploads nach addr kopieren
// 
inline
bool BootFlash::WriteFirstWord(uint32_t addr)
{
  return writeByteArray(addr, firstBytes, 2, true);
}


//
// Erstes Wort des Codes bei addr holen für WriteFirstWord
// 
inline
bool BootFlash::ReadFirstWord(uint32_t addr)
{
  return readByteArray(addr, firstBytes, 2, false);
}


extern BootFlash bootFlash;

#endif // SPIFLASH_CS

#endif
